@@ -12,33 +12,42 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import json
+import ast
 import copy
-import logging
-
+import json
+import netaddr
 import re
+
+from django.core.urlresolvers import reverse
+from django.core import validators as django_validator
 from django import forms
-from django.core.validators import RegexValidator, validate_ipv4_address
-from netaddr import all_matching_cidrs, IPNetwork, IPAddress
+from django.http import Http404
+from django.template import defaultfilters
+from django.utils import html
 from django.utils.translation import ugettext_lazy as _
-from django.utils.encoding import smart_text
-from muranodashboard.environments import api
-from horizon import exceptions, messages
+from horizon import exceptions
+from horizon import forms as hz_forms
+from horizon import messages
 from openstack_dashboard.api import glance
-from openstack_dashboard.api.nova import novaclient
-from django.template.defaultfilters import pluralize
-import horizon.tables as tables
-import horizon.forms as hz_forms
-import floppyforms
-from django.template.loader import render_to_string
+from openstack_dashboard.api import nova
+from oslo_log import log as logging
+import yaql
+
 from muranoclient.common import exceptions as muranoclient_exc
+from muranodashboard.api import packages as pkg_api
+from muranodashboard.common import net
+from muranodashboard.environments import api as env_api
+
+from oslo_log import versionutils
 
 
 LOG = logging.getLogger(__name__)
 
 
 def with_request(func):
-    """The decorator is meant to be used together with `UpdatableFieldsForm':
+    """Injects request into func
+
+    The decorator is meant to be used together with `UpdatableFieldsForm':
     apply it to the `update' method of fields inside that form.
     """
     def update(self, initial, request=None, **kwargs):
@@ -60,14 +69,16 @@ def with_request(func):
     return update
 
 
-def make_yaql_validator(field, form, key, validator_property):
+def make_yaql_validator(validator_property):
+    """Field-level validator uses field's value as its '$' root object."""
+    expr = validator_property['expr'].spec
+    message = validator_property.get('message', '')
+
     def validator_func(value):
-        data = getattr(form, 'cleaned_data', {})
-        data[key] = value
-        form.service.update_cleaned_data(data, form=form)
-        if not validator_property['expr'].__get__(field):
-            raise forms.ValidationError(
-                _(validator_property.get('message', '')))
+        context = yaql.create_context()
+        context['$'] = value
+        if not expr.evaluate(context=context):
+            raise forms.ValidationError(message)
 
     return validator_func
 
@@ -75,7 +86,7 @@ def make_yaql_validator(field, form, key, validator_property):
 def get_regex_validator(expr):
     try:
         validator = expr['validators'][0]
-        if isinstance(validator, RegexValidator):
+        if isinstance(validator, django_validator.RegexValidator):
             return validator
     except (TypeError, KeyError, IndexError):
         pass
@@ -88,19 +99,20 @@ def wrap_regex_validator(validator, message):
     def _validator(value):
         try:
             validator(value)
-        except forms.ValidationError as e:
-            e.messages = [message]
-            delattr(e, 'code')
-            raise e
+        except forms.ValidationError:
+            # provide our own message
+            raise forms.ValidationError(message)
     return _validator
 
 
 def get_murano_images(request):
     images = []
     try:
-        # public filter removed
-        images, _more = glance.image_list_detailed(request)
-    except:
+        # https://bugs.launchpad.net/murano/+bug/1339261 - glance
+        # client version change alters the API. Other tuple values
+        # are _more and _prev (in recent glance client)
+        images = glance.image_list_detailed(request)[0]
+    except Exception:
         LOG.error("Error to request image list from glance ")
         exceptions.handle(request, _("Unable to retrieve public images."))
     murano_images = []
@@ -137,6 +149,9 @@ class RawProperty(object):
         return property(_get, _set, _del)
 
 
+FIELD_ARGS_TO_ESCAPE = ['help_text', 'initial', 'description', 'label']
+
+
 class CustomPropertiesField(forms.Field):
     def __init__(self, description=None, description_title=None,
                  *args, **kwargs):
@@ -144,6 +159,10 @@ class CustomPropertiesField(forms.Field):
         self.description_title = (description_title or
                                   unicode(kwargs.get('label', '')))
 
+        for arg in FIELD_ARGS_TO_ESCAPE:
+            if kwargs.get(arg):
+                kwargs[arg] = html.escape(unicode(kwargs[arg]))
+
         validators = []
         for validator in kwargs.get('validators', []):
             if hasattr(validator, '__call__'):  # single regexpValidator
@@ -194,7 +213,8 @@ class PasswordField(CharField):
     password_re = re.compile('^.*(?=.*\d)(?=.*[a-z])(?=.*[A-Z])(?=.*[%s]).*$'
                              % special_characters)
     has_clone = False
-    validate_password = RegexValidator(
+    original = True
+    validate_password = django_validator.RegexValidator(
         password_re, _('The password must contain at least one letter, one   \
                                number and one special character'), 'invalid')
 
@@ -203,15 +223,15 @@ class PasswordField(CharField):
         return name + '-clone'
 
     def compare(self, name, form_data):
-        if self.is_original() and self.required:
+        if self.original and self.required:
             # run compare only for original fields
             # do not run compare for hidden fields (they are not required)
             if form_data.get(name) != form_data.get(self.get_clone_name(name)):
-                raise forms.ValidationError(_(u"{0}{1} don't match".format(
-                    self.label, pluralize(2))))
+                raise forms.ValidationError(_(u"{0}{1} don't match").format(
+                    self.label, defaultfilters.pluralize(2)))
 
     class PasswordInput(forms.PasswordInput):
-        class Media:
+        class Media(object):
             js = ('muranodashboard/js/passwordfield.js',)
 
     def __init__(self, label, *args, **kwargs):
@@ -244,13 +264,11 @@ class PasswordField(CharField):
         result.error_messages = copy.deepcopy(self.error_messages)
         return result
 
-    def is_original(self):
-        return hasattr(self, 'original') and self.original
-
     def clone_field(self):
         self.has_clone = True
+
         field = copy.deepcopy(self)
-        self.original = True
+        field.original = False
         field.label = _('Confirm password')
         field.error_messages['required'] = _('Please confirm your password')
         field.help_text = _('Retype your password')
@@ -261,229 +279,99 @@ class IntegerField(forms.IntegerField, CustomPropertiesField):
     pass
 
 
-class Column(tables.Column):
-    template_name = 'common/form-fields/data-grid/input.html'
-
-    def __init__(self, transform, table_name=None, **kwargs):
-        if hasattr(self, 'template_name'):
-            def _transform(datum):
-                context = {'data': getattr(datum, self.name, None),
-                           'row_index': str(datum.id),
-                           'table_name': table_name,
-                           'column_name': self.name}
-                return render_to_string(self.template_name, context)
-            _transform.__name__ = transform
-            transform = _transform
-        super(Column, self).__init__(transform, **kwargs)
-
-
-class CheckColumn(Column):
-    template_name = 'common/form-fields/data-grid/checkbox.html'
-
-
-class RadioColumn(Column):
-    template_name = 'common/form-fields/data-grid/radio.html'
-
-
-# FixME: we need to have separated object until find out way to use the same
-# code for MS SQL Cluster datagrid
-class Object(object):
-    def __init__(self, id, **kwargs):
-        self.id = id
-        for key, value in kwargs.iteritems():
-            setattr(self, key, value)
-
-    def as_dict(self):
-        item = {}
-        for key, value in self.__dict__.iteritems():
-            if key != 'id':
-                item[key] = value
-        return item
-
-
-def DataTableFactory(name, columns):
-    class Object(object):
-        row_name_re = re.compile(r'.*\{0}.*')
-
-        def __init__(self, id, **kwargs):
-            self.id = id
-            for key, value in kwargs.iteritems():
-                if isinstance(value, basestring) and \
-                        re.match(self.row_name_re, value):
-                    setattr(self, key, value.format(id))
-                else:
-                    setattr(self, key, value)
-
-    class DataTableBase(tables.DataTable):
-        def __init__(self, request, data, **kwargs):
-            if len(data) and isinstance(data[0], dict):
-                objects = [Object(i, **item)
-                           for (i, item) in enumerate(data, 1)]
-            else:
-                objects = data
-            super(DataTableBase, self).__init__(request, objects, **kwargs)
-
-    class Meta:
-        template = 'common/form-fields/data-grid/data_table.html'
-        name = ''
-        footer = False
-
-    attrs = dict((col_id, cls(col_id, verbose_name=col_name, table_name=name))
-                 for (col_id, cls, col_name) in columns)
-    attrs['Meta'] = Meta
-    return tables.base.DataTableMetaclass('DataTable', (DataTableBase,), attrs)
-
-
-class TableWidget(floppyforms.widgets.Input):
-    template_name = 'common/form-fields/data-grid/table_field.html'
-    delimiter_re = re.compile('([\w-]*)@@([0-9]*)@@([\w-]*)')
-    types = {'label': Column,
-             'radio': RadioColumn,
-             'checkbox': CheckColumn}
-
-    def __init__(self, columns_spec=None, table_class=None, js_buttons=True,
-                 min_value=None, max_value=None, max_sync=None,
-                 *args, **kwargs):
-        assert columns_spec is not None or table_class is not None
-        self.columns = []
-        if columns_spec:
-            for spec in columns_spec:
-                name = spec['column_name']
-                self.columns.append((name,
-                                     self.types[spec['column_type']],
-                                     spec.get('title', None) or name.title()))
-        self.table_class = table_class
-        self.js_buttons = js_buttons
-        self.min_value = min_value
-        self.max_value = max_value
-        self.max_sync = max_sync
-        # FixME: we need to use this hack because TableField passes all kwargs
-        # to TableWidget
-        for kwarg in ('widget', 'description', 'description_title'):
-            kwargs.pop(kwarg, None)
-        super(TableWidget, self).__init__(*args, **kwargs)
-
-    def get_context(self, name, value, attrs=None):
-        ctx = super(TableWidget, self).get_context_data()
-        if value:
-            if self.table_class:
-                cls = self.table_class
-            else:
-                cls = DataTableFactory(name, self.columns)
-            ctx.update({
-                'data_table': cls(self.request, value),
-                'js_buttons': self.js_buttons,
-                'min_value': self.min_value,
-                'max_value': self.max_value,
-                'max_sync': self.max_sync
-            })
-        return ctx
-
-    def value_from_datadict(self, data, files, name):
-        def extract_value(row_key, col_id, col_cls):
-            if col_cls == CheckColumn:
-                val = data.get("{0}@@{1}@@{2}".format(name, row_key, col_id),
-                               False)
-                return val and val == 'on'
-            elif col_cls == RadioColumn:
-                row_id = data.get("{0}@@@@{1}".format(name, col_id), False)
-                return row_id == row_key
-            else:
-                return data.get("{0}@@{1}@@{2}".format(
-                    name, row_key, col_id), None)
-
-        def extract_keys():
-            keys = set()
-            regexp = re.compile('^{name}@@([^@]*)@@.*$'.format(name=name))
-            for key in data.iterkeys():
-                match = re.match(regexp, key)
-                if match and match.group(1):
-                    keys.add(match.group(1))
-            return keys
-
-        items = []
-        if self.table_class:
-            columns = [(_name, column.__class__, unicode(column.verbose_name))
-                       for (_name, column)
-                       in self.table_class.base_columns.items()]
-        else:
-            columns = self.columns
-
-        for row_key in extract_keys():
-            item = {}
-            for column_id, column_instance, column_name in columns:
-                value = extract_value(row_key, column_id, column_instance)
-                item[column_id] = value
-            items.append(Object(row_key, **item))
-
-        return items
-
-    class Media:
-        css = {'all': ('muranodashboard/css/tablefield.css',)}
-
-
-class TableField(CustomPropertiesField):
-    def __init__(self, columns=None, label=None, table_class=None,
-                 initial=None,
-                 **kwargs):
-        widget = TableWidget(columns, table_class, **kwargs)
-        super(TableField, self).__init__(
-            label=label, widget=widget, initial=initial)
-
-    @with_request
-    def update(self, request, **kwargs):
-        self.widget.request = request
-
-    def clean(self, objects):
-        return [obj.as_dict() for obj in objects]
-
-
 class ChoiceField(forms.ChoiceField, CustomPropertiesField):
     pass
 
 
-class DomainChoiceField(ChoiceField):
-    @with_request
-    def update(self, request, environment_id, **kwargs):
-        self.choices = [("", "Not in domain")]
-        domains = api.service_list_by_type(request, environment_id,
-                                           'activeDirectory')
-        self.choices.extend(
-            [(domain.name, domain.name) for domain in domains])
+class DynamicChoiceField(hz_forms.DynamicChoiceField, CustomPropertiesField):
+    pass
 
 
 class FlavorChoiceField(ChoiceField):
+    def __init__(self, *args, **kwargs):
+        if 'requirements' in kwargs:
+            self.requirements = kwargs.pop('requirements')
+        super(FlavorChoiceField, self).__init__(*args, **kwargs)
+
     @with_request
     def update(self, request, **kwargs):
-        self.choices = [(flavor.name, flavor.name) for flavor in
-                        novaclient(request).flavors.list()]
-        for flavor in self.choices:
-            if 'medium' in flavor[1]:
-                self.initial = flavor[0]
+        self.choices = []
+        flavors = nova.novaclient(request).flavors.list()
+
+        # If no requirements are present, return all the flavors.
+        if not hasattr(self, 'requirements'):
+            self.choices = [(flavor.name, flavor.name) for flavor in flavors]
+        else:
+            for flavor in flavors:
+                # If a flavor doesn't meet a minimum requirement,
+                # do not add it to the options list and skip to the
+                # next flavor.
+                if flavor.vcpus < self.requirements.get('min_vcpus', 0):
+                    continue
+                if flavor.disk < self.requirements.get('min_disk', 0):
+                    continue
+                if flavor.ram < self.requirements.get('min_memory_mb', 0):
+                    continue
+                self.choices.append((flavor.name, flavor.name))
+        # Search through selected flavors
+        for flavor_name, flavor_name in self.choices:
+            if 'medium' in flavor_name:
+                self.initial = flavor_name
                 break
 
 
-class KeyPairChoiceField(ChoiceField):
+class KeyPairChoiceField(DynamicChoiceField):
     " This widget allows to select keypair for VMs "
     @with_request
     def update(self, request, **kwargs):
         self.choices = [('', _('No keypair'))]
-        for keypair in novaclient(request).keypairs.list():
+        for keypair in nova.novaclient(request).keypairs.list():
             self.choices.append((keypair.name, keypair.name))
 
 
+# NOTE(kzaitsev): for transform to work correctly on horizon SelectWidget
+# Choice has to be non-string
+class Choice(object):
+    """A choice that allows disabling specific choices in a SelectWidget."""
+    def __init__(self, title, enabled):
+        self.title = title
+        self.enabled = enabled
+
+
+def _get_title(data):
+    if isinstance(data, Choice):
+        return data.title
+    return data
+
+
+def _disable_non_ready(data):
+    if getattr(data, 'enabled', True):
+        return {}
+    else:
+        return {'disabled': 'disabled'}
+
+
 class ImageChoiceField(ChoiceField):
+    widget = hz_forms.SelectWidget(transform=_get_title,
+                                   transform_html_attrs=_disable_non_ready)
+
     def __init__(self, *args, **kwargs):
         self.image_type = kwargs.pop('image_type', None)
         super(ImageChoiceField, self).__init__(*args, **kwargs)
 
     @with_request
     def update(self, request, **kwargs):
-        image_mapping, image_choices = {}, []
+        image_map, image_choices = {}, []
         murano_images = get_murano_images(request)
         for image in murano_images:
             murano_data = image.murano_property
             title = murano_data.get('title', image.name)
+
+            if image.status == 'active':
+                title = Choice(title, enabled=True)
+            else:
+                title = Choice("{} ({})".format(title, image.status),
+                               enabled=False)
             if self.image_type is not None:
                 itype = murano_data.get('type')
 
@@ -494,10 +382,10 @@ class ImageChoiceField(ChoiceField):
                 if (not itype.startswith(prefix) and
                         not self.image_type == itype):
                     continue
-            image_mapping[smart_text(title)] = image.name
+            image_map[image.id] = title
 
-        for name in sorted(image_mapping.keys()):
-            image_choices.append((image_mapping[name], name))
+        for id_, title in sorted(image_map.iteritems(), key=lambda e: e[1]):
+            image_choices.append((id_, title))
         if image_choices:
             image_choices.insert(0, ("", _("Select Image")))
         else:
@@ -506,13 +394,57 @@ class ImageChoiceField(ChoiceField):
         self.choices = image_choices
 
 
+class NetworkChoiceField(ChoiceField):
+    def __init__(self,
+                 include_subnets=True,
+                 filter=None,
+                 murano_networks=None,
+                 allow_auto=True,
+                 *args,
+                 **kwargs):
+        self.filter = filter
+        if murano_networks:
+            if murano_networks.lower() not in ["exclude", "translate"]:
+                raise ValueError(_("Invalid value of 'murano_nets' option"))
+        self.murano_networks = murano_networks
+        self.include_subnets = include_subnets
+        self.allow_auto = allow_auto
+        super(NetworkChoiceField, self).__init__(*args,
+                                                 **kwargs)
+
+    @with_request
+    def update(self, request, **kwargs):
+        """Populates available networks in the control
+
+        This method is called automatically when the form which contains it is
+        rendered
+        """
+        network_choices = net.get_available_networks(request,
+                                                     self.include_subnets,
+                                                     self.filter,
+                                                     self.murano_networks)
+        if self.allow_auto:
+            network_choices.insert(0, ((None, None), _('Auto')))
+        self.choices = network_choices or []
+
+    def to_python(self, value):
+        """Converts string representation of widget to tuple value
+
+        Is called implicitly during form cleanup phase
+        """
+        if value:
+            return ast.literal_eval(value)
+        else:  # may happen if no networks are available and "Auto" is disabled
+            return None, None
+
+
 class AZoneChoiceField(ChoiceField):
     @with_request
     def update(self, request, **kwargs):
         try:
-            availability_zones = novaclient(request).availability_zones.\
-                list(detailed=False)
-        except:
+            availability_zones = nova.novaclient(
+                request).availability_zones.list(detailed=False)
+        except Exception:
             availability_zones = []
             exceptions.handle(request,
                               _("Unable to retrieve  availability zones."))
@@ -534,12 +466,14 @@ class BooleanField(forms.BooleanField, CustomPropertiesField):
         else:
             widget = forms.CheckboxInput(attrs={'class': 'checkbox'})
         kwargs['widget'] = widget
+        kwargs['required'] = False
         super(BooleanField, self).__init__(*args, **kwargs)
 
 
-# This type of field is deprecated, in future floating ip fields should be
-# implemented using a regular BooleanField
-# TODO(ativelkov, tsufiev): add a deprecated decorator
+@versionutils.deprecated(
+    as_of=versionutils.deprecated.JUNO,
+    in_favor_of='type boolean (regular BooleanField)',
+    remove_in=1)
 class FloatingIpBooleanField(BooleanField):
     pass
 
@@ -552,12 +486,12 @@ class ClusterIPField(CharField):
     @staticmethod
     def make_nova_validator(request, ip_ranges):
         def perform_checking(ip):
-            validate_ipv4_address(ip)
-            if not all_matching_cidrs(ip, ip_ranges) and ip_ranges:
+            django_validator.validate_ipv4_address(ip)
+            if not netaddr.all_matching_cidrs(ip, ip_ranges) and ip_ranges:
                 raise forms.ValidationError(_('Specified Cluster Static IP is'
                                               ' not in valid IP range'))
             try:
-                ip_info = novaclient(request).fixed_ips.get(ip)
+                ip_info = nova.novaclient(request).fixed_ips.get(ip)
             except exceptions.UNAUTHORIZED:
                 LOG.error("Error to get information about IP address"
                           " using novaclient")
@@ -569,7 +503,7 @@ class ClusterIPField(CharField):
                 msg = "Could not found fixed ips for ip %s" % (ip,)
                 LOG.error(msg)
                 exceptions.handle(
-                    request, _(msg),
+                    request, msg,
                     ignore=True)
             else:
                 if ip_info.hostname:
@@ -578,21 +512,22 @@ class ClusterIPField(CharField):
         return perform_checking
 
     def update_network_params(self, request, environment_id):
-        env = api.environment_get(request, environment_id)
+        env = env_api.environment_get(request, environment_id)
         self.existing_subnet = env.networking.get('cidr')
         self.network_topology = env.networking.get('topology')
 
     def make_neutron_validator(self):
         def perform_checking(ip):
-            validate_ipv4_address(ip)
+            django_validator.validate_ipv4_address(ip)
             if not self.existing_subnet:
                 raise forms.ValidationError(
                     _('Cannot get allowed subnet for the environment, '
                       'consult your admin'))
-            elif not IPAddress(ip) in IPNetwork(self.existing_subnet):
+            elif not netaddr.IPAddress(ip) in netaddr.IPNetwork(
+                    self.existing_subnet):
                 raise forms.ValidationError(
                     _('Specified IP address should belong to {0} '
-                      'subnet'.format(self.existing_subnet)))
+                      'subnet').format(self.existing_subnet))
 
         return perform_checking
 
@@ -602,13 +537,14 @@ class ClusterIPField(CharField):
 
         if self.network_topology == 'nova':
             try:
-                network_list = novaclient(request).networks.list()
+                network_list = nova.novaclient(request).networks.list()
                 ip_ranges = [network.cidr for network in network_list]
                 ranges = ', '.join(ip_ranges)
             except StandardError:
                 ip_ranges, ranges = [], ''
             if ip_ranges:
-                self.help_text = _('Select IP from available range: ' + ranges)
+                self.help_text = _('Select IP from '
+                                   'available range: {0} ').format(ranges)
             else:
                 self.help_text = _('Specify valid fixed IP')
             self.validators = [self.make_nova_validator(request, ip_ranges)]
@@ -620,15 +556,16 @@ class ClusterIPField(CharField):
             self.validators = [self.make_neutron_validator()]
         else:  # 'flat' topology
             raise NotImplementedError('Flat topology is not implemented yet')
-        self.error_messages['invalid'] = validate_ipv4_address.message
+        self.error_messages['invalid'] = \
+            django_validator.validate_ipv4_address.message
 
 
 class DatabaseListField(CharField):
-    validate_mssql_identifier = RegexValidator(
+    validate_mssql_identifier = django_validator.RegexValidator(
         re.compile(r'^[a-zA-z_][a-zA-Z0-9_$#@]*$'),
-        _((u'First symbol should be latin letter or underscore. Subsequent ' +
-           u'symbols can be latin letter, numeric, underscore, at sign, ' +
-           u'number sign or dollar sign')))
+        _(u'First symbol should be latin letter or underscore. Subsequent '
+          u'symbols can be latin letter, numeric, underscore, at sign, '
+          u'number sign or dollar sign'))
 
     default_error_messages = {'invalid': validate_mssql_identifier.message}
 
@@ -645,22 +582,71 @@ class DatabaseListField(CharField):
             self.validate_mssql_identifier(db_name)
 
 
-def make_select_cls(fqn):
+def make_select_cls(fqns):
+    if not isinstance(fqns, (tuple, list)):
+        fqns = (fqns,)
+
+    class Widget(hz_forms.fields.DynamicSelectWidget):
+        def __init__(self, attrs=None, **kwargs):
+            if attrs is None:
+                attrs = {'class': 'murano_add_select'}
+            else:
+                attrs.setdefault('class', '')
+                attrs['class'] += ' murano_add_select'
+            super(Widget, self).__init__(attrs=attrs, **kwargs)
+
+        class Media(object):
+            js = ('muranodashboard/js/add-select.js',)
+
     class DynamicSelect(hz_forms.DynamicChoiceField, CustomPropertiesField):
-        def __init__(self, *args, **kwargs):
+        widget = Widget
+
+        def __init__(self, empty_value_message=None, *args, **kwargs):
             super(DynamicSelect, self).__init__(*args, **kwargs)
-            self.widget.add_item_link = 'horizon:murano:catalog:add_many'
+            if empty_value_message is not None:
+                self.empty_value_message = empty_value_message
+            else:
+                self.empty_value_message = _('Select Application')
 
         @with_request
         def update(self, request, environment_id, **kwargs):
-            app_id = api.app_id_by_fqn(request, fqn)
-            if app_id is None:
-                msg = "Application with FQN='{0}' doesn't exist"
-                raise KeyError(msg.format(fqn))
-            self.widget.add_item_link_args = (
-                environment_id, app_id, False, True)
-            self.choices = [('', _('Select Application'))]
-            apps = api.service_list_by_fqn(request, environment_id, fqn)
-            self.choices.extend([(app['?']['id'], app.name) for app in apps])
+            def _make_link():
+                ns_url = 'horizon:murano:catalog:add'
+
+                def _reverse(_fqn):
+                    _app = pkg_api.app_by_fqn(request, _fqn)
+                    if _app is None:
+                        msg = "Application with FQN='{0}' doesn't exist"
+                        messages.error(request, msg.format(_fqn))
+                        raise Http404(msg.format(_fqn))
+                    args = (_app.id, environment_id, False, True)
+                    return _app.name, reverse(ns_url, args=args)
+                return json.dumps([_reverse(cls) for cls in fqns])
+
+            self.widget.add_item_link = _make_link
+            apps = env_api.service_list_by_fqns(request, environment_id, fqns)
+            choices = [('', self.empty_value_message)]
+            choices.extend([(app['?']['id'],
+                             html.escape(app.name)) for app in apps])
+            self.choices = choices
+            # NOTE(tsufiev): streamline the drop-down UX: auto-select the
+            # single available option in a drop-down
+            if len(choices) == 2:
+                self.initial = choices[1][0]
+
+        def clean(self, value):
+            value = super(DynamicSelect, self).clean(value)
+            return None if value == '' else value
 
     return DynamicSelect
+
+
+@versionutils.deprecated(
+    as_of=versionutils.deprecated.JUNO,
+    in_favor_of='type io.murano.windows.ActiveDirectory with a custom '
+                'emptyValueMessage attribute',
+    remove_in=1)
+class DomainChoiceField(make_select_cls('io.murano.windows.ActiveDirectory')):
+    def __init__(self, *args, **kwargs):
+        super(DomainChoiceField, self).__init__(*args, **kwargs)
+        self.choices = [('', _('Not in domain'))]
