@@ -41,6 +41,7 @@ from oslo_log import log as logging
 from muranodashboard import api
 from muranodashboard.api import packages as pkg_api
 from muranodashboard.catalog import views as catalog_views
+from muranodashboard import client_overrides
 from muranodashboard.common import utils as muranodashboard_utils
 from muranodashboard.environments import consts
 from muranodashboard.packages import consts as packages_consts
@@ -55,6 +56,8 @@ FORMS = [('upload', forms.ImportPackageForm),
 
 BUNDLE_FORMS = [('upload', forms.ImportBundleForm), ]
 
+client_overrides.override()
+
 
 def is_app(wizard):
     """Check if we're uploading an application
@@ -139,7 +142,7 @@ class PackageDefinitionsView(horizon_tables.DataTableView):
                 if package.owner_id == current_tenant['id']:
                     packages[i].tenant_name = current_tenant['name']
                 else:
-                    packages[i].tenant_name = _('UNKNOWN')
+                    packages[i].tenant_name = 'other'
         return packages
 
     def get_context_data(self, **kwargs):
@@ -242,15 +245,11 @@ class ImportBundleWizard(views.ModalFormMixin,
                             image_specs=dep_package.images(),
                             base_url=base_url)
                         for img in imgs:
-                            msg = _("Trying to add {0} image to glance. "
-                                    "Image will be ready for deployment after"
-                                    " successful upload").format(img['name'],)
-                            messages.warning(self.request, msg)
-                            log_msg = _("Trying to add {0}, {1} image to "
-                                        "glance. Image will be ready for "
-                                        "deployment after successful upload")\
-                                .format(img['name'], img['id'],)
-                            LOG.info(log_msg)
+                            msg = _("Added {0}, {1} image to glance").format(
+                                img['name'], img['id'],
+                            )
+                            messages.success(self.request, msg)
+                            LOG.info(msg)
                     except Exception as e:
                         msg = _("Error {0} occurred while installing "
                                 "images for {1}").format(e, dep_name)
@@ -432,15 +431,11 @@ class ImportPackageWizard(views.ModalFormMixin,
                         image_specs=package.images(),
                         base_url=base_url)
                     for img in imgs:
-                        msg = _("Trying to add {0} image to glance. "
-                                "Image will be ready for deployment after "
-                                "successful upload").format(img['name'],)
-                        messages.warning(self.request, msg)
-                        log_msg = _("Trying to add {0}, {1} image to "
-                                    "glance. Image will be ready for "
-                                    "deployment after successful upload")\
-                            .format(img['name'], img['id'],)
-                        LOG.info(log_msg)
+                        msg = _("Added {0}, {1} image to glance").format(
+                            img['name'], img['id'],
+                        )
+                        messages.success(self.request, msg)
+                        LOG.info(msg)
                 except Exception as e:
                     msg = _("Error {0} occurred while installing "
                             "images for {1}").format(e, name)
